import pygame
import data
import env
import utils

class Building:
    def __init__(self, filename = None, editing=0):
        self.floors = []
        self.walls = []
        self.name = "unnamed"
        self.start = (1, 1)
        self.end = (10, 1)
        self.won = 0
        self.editing = editing
        
        if filename:
            self.load_data(filename)

        self.lvltimer = 0
        self.timer_active = 0

    def reset_timer(self):
        self.lvltimer = 0
        self.timer_active = 1

    def tick_timer(self):
        if self.timer_active:
            self.lvltimer += 1.0/60 #tick 1/60 of a second / frame

    def get_timer(self):
        return self.lvltimer
        
    def stop_timer(self):
        self.timer_active = 0

    def win_level(self):
        if not self.editing:
            self.stop_timer()
            print "won at time",self.lvltimer
        self.won = 1

    def save_data(self):
        lvlnum = 1
        while os.path.exists("lvl%d" % lvlnum):
            lvlnum += 1
        filename = "lvl%d" % lvlnum
        f = open(filename, "w")
        f.write("name:%s\n" % self.name)
        f.write("start:%d %d\n" % self.start)
        f.write("end:%d %d\n" % self.end)

        for wall in self.walls:
            f.write("wall:%d %d %d\n" % (wall.x, wall.y, wall.height))

        for floor in self.floors:
            if floor.trampoline:
                f.write("trampoline:%d %d %d\n" % (floor.y, floor.startx, floor.stopx))
            else:
                f.write("floor:%d %d %d\n" % (floor.y, floor.startx, floor.stopx))
        f.close()

    def set_start(self, x, y):
        self.start = (x,y)

    def set_end(self, x, y):
        self.end = (x,y)

    def get_intersection_below(self, x, y):
        cur_y = 0
        intersect_floor = None
        for floor in self.floors:
            if floor.intersect(x) and floor.y <= y:
                if floor.y >= cur_y:
                    cur_y = floor.y
                    intersect_floor = floor
        #print cur_y
        return intersect_floor

    def get_intersection_above(self, x, y):
        cur_y = 1000
        intersect_floor = None
        for floor in self.floors:
            if floor.intersect(x) and floor.y > y:
                if floor.y <= cur_y:
                    cur_y = floor.y
                    intersect_floor = floor
        #print cur_y
        return intersect_floor

    def get_intersecting_wall(self, old_x, old_y, new_x, new_y):
        for wall in self.walls:
            if wall.intersect(old_x, old_y, new_x, new_y):
                return wall
        return None

    def get_wall_under(self, x, y):
        for wall in self.walls:
            if wall.is_under_object(x,y):
                return wall
        return None

    def get_wall_over(self, x, y):
        for wall in self.walls:
            if wall.is_over_object(x,y):
                return wall
        return None

    def does_sideway_collision(self, old_x, old_y, x, y):
        for floor in self.floors:
            if floor.does_sideway_collision_with(old_x, old_y, x, y):
                return True
        return False
                

    def make_floor(self, floor):
        self.floors.append(floor)

    def make_wall(self, wall):
        self.walls.append(wall)
                
    def draw_floors(self, surface):
        for floor in self.floors:
            floor.draw(surface)

    def draw_walls(self, surface):
        for wall in self.walls:
            wall.draw(surface)

    def draw(self, surface):
        self.draw_floors(surface)
        self.draw_walls(surface)
        self.draw_start_and_end(surface)

    def kill_at(self, x, y):
        for wall in self.walls:
            if wall.x == x:
                if wall.y <= y and (wall.y+wall.height) >= y:
                    self.walls.pop(self.walls.index(wall))
        for floor in self.floors:
            if floor.y == y:
                if floor.startx <= x and floor.stopx >= x:
                    self.floors.pop(self.floors.index(floor))
                

    def draw_start_and_end(self, surface):

        #start!
        pygame.draw.circle(surface, 0x00ff00, 
                           utils.to_real_coords(self.start), \
                           5, 4)

        pygame.draw.circle(surface, 0xff0000, 
                           utils.to_real_coords(self.end), \
                           5, 4)



        #end !

    def load_data(self, filename):
        f = data.load(filename)
        for line in f:
            
            if line.startswith("name"):
                self.name = line.split(":")[1].strip()
            else:
                args = map(int,line.split(":")[1].split(" "))
                if line.startswith("start"):
                    self.set_start(args[0], args[1])
                elif line.startswith("end"):
                    self.set_end(args[0], args[1])
                elif line.startswith("wall"):
                    self.make_wall(env.Wall(args[0], args[1], args[2]))
                elif line.startswith("floor"):
                    self.make_floor(env.Floor(args[0], args[1], args[2]))
                elif line.startswith("trampoline"):
                    self.make_floor(env.Trampoline(args[0], args[1], args[2]))
            
            
