import pygame, sys, os
import data
import fontmanager

import building, env, gameobjects, utils

from pygame.locals import *

class DashManDX:
    def __init__(self):
        #pygames stuff
        pygame.init()
        self.SIZE = SCREENW, SCREENH = 640, 480
        self.screen = pygame.display.set_mode(self.SIZE)
        pygame.display.set_caption('Dashman DX')
        pygame.mouse.set_visible(1)
        self.clock = pygame.time.Clock()

        self.mode = 0 # 0 = mainmenu

        self.mainmenu = MainMenu(self)

    def show_main_menu(self):
        self.mode = 0
        self.mainmenu.mainloop()

    def start_main_game(self):
        self.mode = 1
        #game = Game()
        self.play_level("lvl9")
        if self.mode == 1:
            self.play_level("lvl9")
        if self.mode == 1:
            self.play_level("lvl9")
        self.mode = 0

    def play_level(self, lvlname):
        game = Game()
        game.play_level(lvlname, self)


    def play_custom_levels(self):
        pass

    def start_level_editor(self):
        self.mode = 2
        editor = LevelEditor(self)
        editor.mainloop()

    def quit_game(self):
        sys.exit(0)


class MainMenu:
    class MenuOption:
        def __init__(self, name, function):
            self.name = name
            self.function = function
    
    def __init__(self, mainapp):
        self.mainapp = mainapp
        self.fontmgr = fontmanager.mFontManager((('Caveman.ttf', 24),('Caveman.ttf', 36)))
        self.background = data.load_image("bg.jpg")[0].convert()

        self.menu = []
        self.menu.append(self.MenuOption("Start Game", DashManDX.start_main_game))
        self.menu.append(self.MenuOption("Play custom level", DashManDX.play_custom_levels))
        self.menu.append(self.MenuOption("Level editor", DashManDX.start_level_editor))
        self.menu.append(self.MenuOption("Quit Game", DashManDX.quit_game))
                
        self.choice = 0
        
    def mainloop(self):
        while self.mainapp.mode == 0:
            self.mainapp.clock.tick(60)
            self.handle_input()

            self.draw(self.mainapp.screen)
            
    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE: sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    pass
                
                elif event.key == pygame.K_UP:
                    self.choice -= 1
                    if self.choice < 0:
                        self.choice = 3
                elif event.key == pygame.K_DOWN:
                    self.choice += 1
                    if self.choice >= 4:
                        self.choice = 0                        
                elif event.key == pygame.K_LEFT:
                    pass
                elif event.key == pygame.K_RIGHT:
                    self.menu[self.choice].function(self.mainapp)


    def draw(self, surface):
        surface.blit(self.background, (0,0))

        x, y = (150,150)
        numitem = 0
        
        for option in self.menu:
            if numitem == self.choice:
                size = 36
                x_off = -20
            else:
                size = 24
                x_off = 0
            self.fontmgr.Draw(surface, 'Caveman.ttf', size, option.name,
                              (x+x_off, y), (150,42,42), 'left', 'center', True)
            y += size+15
            numitem += 1
        
        pygame.display.flip()


class Game:
    def __init__(self):
        pass

    def play_level(self, lvlname, mainapp):
        self.mainapp = mainapp
        self.background = data.load_image("bg.jpg")[0].convert()
        self.building = building.Building(lvlname)
        self.fontmgr = fontmanager.mFontManager((('Caveman.ttf', 24),('Caveman.ttf', 36)))

        herox, heroy = self.building.start
        self.hero = gameobjects.Hero(herox, heroy, self.building, mainapp.screen)

        self.allsprites = pygame.sprite.RenderPlain((self.hero,))

        self.building.reset_timer()

        self.level_start()
        self.mainloop()
        
        
        
    def mainloop(self):
        while self.mainapp.mode == 1:
            self.mainapp.clock.tick(60)
            self.handle_input()
            self.allsprites.update()
            self.building.tick_timer()
            self.draw(self.mainapp.screen)

            if self.building.won:
                self.level_complete()
                break

    def level_start(self):
        #stuff to do at start of level
        #display things and so on

        tickstoshow = 60*1.5 #1.5 seconds
        while tickstoshow:
            self.mainapp.clock.tick(60)
            self.allsprites.update()
            self.draw(self.mainapp.screen, self.building.name, "start!")
            tickstoshow -= 1



    def level_complete(self):
        tickstoshow = 60*3 #3 seconds
        while tickstoshow:
            self.mainapp.clock.tick(60)
            #self.allsprites.update()
            
            self.draw(self.mainapp.screen, "level complete","%.2f seconds" % self.building.get_timer())
            tickstoshow -= 1

        

    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    print "test"
                    self.mainapp.mode = 0 #sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    self.hero.dash()
                elif event.key == pygame.K_UP:
                    self.hero.moveUp()
                elif event.key == pygame.K_LEFT:
                    self.hero.moveLeft()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveRight()

            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    self.hero.moveStop()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveStop()

    def draw(self, surface, text1="", text2=""):
        surface.blit(self.background, (0, 0))

        #grid.draw(surface)
        self.building.draw(surface)
        self.allsprites.draw(surface)

        if text1:
            self.fontmgr.Draw(self.mainapp.screen, 'Caveman.ttf', 36, text1,
                              pygame.Rect(0,0,640,240), (150,42,42), 'center', 'center', True)
        if text2:
            self.fontmgr.Draw(self.mainapp.screen, 'Caveman.ttf', 36, text2,
                              pygame.Rect(0,120,640,240), (150,42,42), 'center', 'center', True)
        
        pygame.display.flip()


class LevelEditor:
    def __init__(self, mainapp):
        self.mainapp = mainapp
        self.background = data.load_image("bg.jpg")[0].convert()
        self.building = building.Building(editing=1)
        self.fontmgr = fontmanager.mFontManager((('Caveman.ttf', 24),('Caveman.ttf', 36)))

        herox, heroy = self.building.start
        self.hero = gameobjects.Hero(herox, heroy, self.building, mainapp.screen)

        self.allsprites = pygame.sprite.RenderPlain((self.hero,))

        self.building.reset_timer()


        self.grid = utils.Grid()
        self.to_make = 3
        self.last_point = (-1,-1)


        #self.level_start()
        self.mainloop()

    

    def respawn_hero(self):
        herox, heroy = self.building.start
        self.hero.x = herox
        self.hero.y = heroy
        self.hero.v_x = self.hero.v_y = 0

    def mainloop(self):
        while self.mainapp.mode == 2:
            self.mainapp.clock.tick(60)
            self.handle_input()
            self.allsprites.update()
            self.building.tick_timer()
            self.draw(self.mainapp.screen)

            if self.building.won:
                self.respawn_hero()
                self.building.won = 0

    def handle_input(self):
        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    print "test"
                    self.mainapp.mode = 0 #sys.exit(0)
                elif event.key == pygame.K_SPACE:
                    self.hero.dash()
                elif event.key == pygame.K_UP:
                    self.hero.moveUp()
                elif event.key == pygame.K_LEFT:
                    self.hero.moveLeft()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveRight()
                elif event.key == pygame.K_DOWN:
                    self.respawn_hero()
                    

                #LEVEL EDITOR EVENTS
                elif event.key == pygame.K_0: #none
                    self.to_make = 0
                elif event.key == pygame.K_1: #start
                    self.to_make = 1
                elif event.key == pygame.K_2: #end
                    self.to_make = 2
                elif event.key == pygame.K_3: #floor / wall
                    self.to_make = 3
                elif event.key == pygame.K_4: #trampoline
                    self.to_make = 4
                    
            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_LEFT:
                    self.hero.moveStop()
                elif event.key == pygame.K_RIGHT:
                    self.hero.moveStop()

            #MOUSE EVENTS FOR KEY EDITOR!!!
            elif event.type == MOUSEBUTTONDOWN:

                (lb, mb, rb)= pygame.mouse.get_pressed()
                x = (pygame.mouse.get_pos()[0]+8)/16
                y = 20-(pygame.mouse.get_pos()[1]+12)/24
                
                print "clicked:",x,y

                #kill what is right clicked!
                if rb:
                    self.building.kill_at(x,y)

                #CREATION!!!
                if lb:
                    if self.to_make == 1: #start
                        print "setting start!",x,"x",y
                        self.building.set_start(x,y)
                    elif self.to_make == 2: #end
                        print "setting end!",x,"x",y
                        self.building.set_end(x,y)
                    elif self.to_make == 3: #floor / wall
                        #check if any last point
                        if self.last_point == (-1,-1): # no last point
                            self.last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a wall!
                            if (self.last_point[0] == x ) and (self.last_point[1] != y):
                                if self.last_point[1] < y:
                                    w_y = self.last_point[1]
                                    height = y - self.last_point[1]
                                else:
                                    w_y = y
                                    height = self.last_point[1] -y
                                self.building.make_wall(env.Wall(x, w_y, height))

                            #Draw a floor!
                            elif (self.last_point[1] == y) and (self.last_point[0] != x):
                                if self.last_point[0] < x:
                                    x1 = self.last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = self.last_point[0]
                                self.building.make_floor(env.Floor(y, x1, x2))
                            self.last_point = (-1, -1)
                    elif self.to_make == 4: #trampoline
                        if self.last_point == (-1,-1): # no last point
                            self.last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a trampoline floor!
                            if (self.last_point[1] == y) and (self.last_point[0] != x):
                                if self.last_point[0] < x:
                                    x1 = self.last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = self.last_point[0]
                                self.building.make_floor(env.Trampoline(y, x1, x2))
                            self.last_point = (-1, -1)
                        






    def draw(self, surface):
        surface.blit(self.background, (0, 0))

        self.grid.draw(surface)
        self.building.draw(surface)
        self.allsprites.draw(surface)

        
        pygame.display.flip()



def main():
    mainapp = DashManDX()
    
    mainapp.show_main_menu()
    #mainapp.play_level("lvl9")
    



def do_later():

    
    #my stuff
    building = Building("lvl9")

   
    
    herox, heroy = building.start
    hero = Hero(herox, heroy, building, screen)

    allsprites = pygame.sprite.RenderPlain((hero,))

    grid = Grid()
    to_make = 0
    last_point = (-1,-1)

    building.reset_timer()
    
    while(1):
        clock.tick(60)
        #hero.update()
        allsprites.update()
        building.tick_timer()

        #key events
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
		if event.key == pygame.K_ESCAPE: sys.exit(0)
		elif event.key == pygame.K_SPACE:
                    hero.dash()
                
		elif event.key == pygame.K_UP:
                    hero.moveUp()
		elif event.key == pygame.K_LEFT:
                    hero.moveLeft()
		elif event.key == pygame.K_RIGHT:
                    hero.moveRight()
                elif event.key == pygame.K_s:
                    building.save_data()

                elif event.key == pygame.K_0: #none
                    to_make = 1
                elif event.key == pygame.K_1: #start
                    to_make = 1
                elif event.key == pygame.K_2: #end
                    to_make = 2
                elif event.key == pygame.K_3: #floor / wall
                    to_make = 3
                elif event.key == pygame.K_4: #wall
                    to_make = 4
                elif event.key == pygame.K_5: #trampoline
                    to_make = 5
                elif event.key == pygame.K_6:
                    to_make = 6
                elif event.key == pygame.K_7:
                    to_make = 7
                    

            elif event.type == pygame.KEYUP:
                if event.key == pygame.K_DOWN: pass
		elif event.key == pygame.K_UP:
                    pass
		elif event.key == pygame.K_LEFT:
                    hero.moveStop()
		elif event.key == pygame.K_RIGHT:
                    hero.moveStop()
            
            elif event.type == MOUSEBUTTONDOWN:

                (lb, mb, rb)= pygame.mouse.get_pressed()
                x = (pygame.mouse.get_pos()[0]+8)/16
                y = 20-(pygame.mouse.get_pos()[1]+12)/24

                #print pygame.mouse.get_pos()
                print x,y


                if rb:
                    building.kill_at(x,y)

                if lb:
                    if to_make == 1: #start
                        print "setting start!",x,"x",y
                        building.set_start(x,y)
                    elif to_make == 2: #end
                        print "setting end!",x,"x",y
                        building.set_end(x,y)
                    elif to_make == 3: #floor
                        #check if any last point
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a wall!
                            if (last_point[0] == x ) and (last_point[1] != y):
                                if last_point[1] < y:
                                    w_y = last_point[1]
                                    height = y - last_point[1]
                                else:
                                    w_y = y
                                    height = last_point[1] -y
                                building.make_wall(Wall(x, w_y, height))

                            #Draw a floor!
                            elif (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Floor(y, x1, x2))
                            last_point = (-1, -1)
                    elif to_make == 4: #trampoline
                        if last_point == (-1,-1): # no last point
                            last_point = (x,y)
                        else: #last point! draw some stuff!
                            #Draw a trampoline floor!
                            if (last_point[1] == y) and (last_point[0] != x):
                                if last_point[0] < x:
                                    x1 = last_point[0]
                                    x2 = x
                                else:
                                    x1 = x
                                    x2 = last_point[0]
                                building.make_floor(Trampoline(y, x1, x2))
                            last_point = (-1, -1)
                        


                
                    

                
        screen.blit(background, (0, 0))

        #grid.draw(screen)
        building.draw(screen)
        allsprites.draw(screen)

        pygame.display.flip()
